@@ -10,7 +10,11 @@ BEGIN { %HTTP = map { $_ => 1 } qw/get post put delete connect head options trac
 
 sub new {
 	my ($pkg, @args) = @_;
-	my $self = $pkg->SUPER::new(@args);
+	my $self = $pkg->SUPER::new(
+		app => 0,
+		restrict_path => '/',
+		@args
+	);
 	(my $namespace = $pkg) =~ s/^.*Controller:://;
 	$namespace =~ s/\:\:/\//g;
 	$self->namespace = lc( $namespace );
@@ -18,6 +22,25 @@ sub new {
 	return $self;
 }
 
+sub build_terse {
+	my ($self, $t) = @_;
+	if ($self->models) {
+		$t->{model} = sub {
+			my ($t, $model) = @_;
+			$t->raiseError("invalid model: ${model}", 400) unless $self->models->$model;
+			return $t->models->$model ||= $self->models->$model->connect();
+		};
+	}
+	if ($self->controllers) {
+		$t->{controller} = sub {
+			my ($t, $controller) = @_;
+			$t->raiseError("invalid model: ${controller}", 400) unless $self->controllers->{$controller};
+			return $t->controllers->{$controller};
+		};
+	}
+	return $t;
+}
+
 sub MODIFY_CODE_ATTRIBUTES {
 	my ($package, $coderef, @attributes, @disallowed) = @_;
 	my $name = svref_2object($coderef)->GV->NAME;
@@ -61,11 +84,16 @@ sub dispatch {
 	my $package = ref $self || $self;
 	my $dispatcher = $Terse::Controller::dispatcher{$package}{$req};
 	if (!$dispatcher) {
-		$t->logError('Invalid request', 400);
+		$t->logError('Invalid dispatch request', 400);
+		return;
+	}
+	if (!$self->{app} && $t->request->uri->path ne $self->restrict_path) {
+		$t->logError('Invalid path', 500);
 		return;
 	}
 	my $dispatch;
 	DISPATCH: for my $candidate (reverse @{$dispatcher}) {
+		next DISPATCH unless ($candidate->{lc($t->request->method)} || $candidate->{any});
 		if ($candidate->{params}) {
 			for my $param (keys %{$candidate->{params}}) {
 				next DISPATCH if (!$t->params->{$param});
@@ -75,14 +103,8 @@ sub dispatch {
 				);
 			}
 		}
-		if ($candidate->{any}) {
-			$dispatch = $candidate;
-			last;
-		}
-		if ($candidate->{lc($t->request->method)}) {
-			$dispatch = $candidate;
-			last;
-		}
+		$dispatch = $candidate;
+		last;
 	}
 	$dispatch = $self->dispatch_hook($dispatch) if $self->can('dispatch_hook'); 
 	my $callback = $dispatch->{callback};
@@ -91,7 +113,7 @@ sub dispatch {
 		return;
 	}
 	return $self->$callback($t, @params);
-} 
+}
 
 sub _partial_match {
 	my ($self, $param, $spec) = @_;
@@ -124,7 +146,7 @@ Terse::Controller - controllers made simple.
 
 =head1 VERSION
 
-Version 0.100
+Version 0.101
 
 =cut
 
