@@ -2,6 +2,12 @@ package TestAppController;
 
 use base 'Terse::Controller';
 
+use TestAppController::Model;
+
+sub build_controller {
+	$_[0]->models->test = TestAppController::Model->new();
+}
+
 sub hidden {
 
 }
@@ -23,7 +29,7 @@ sub hello :any(hello_world) {
 }
 
 sub hello_get :get(hello_world) {
-	$_[1]->response->hello = "world 2";
+	$_[1]->response->hello = $_[1]->model('test')->something;
 }
 
 sub hello_get_with_params :get(hello_world) :params(test => 'okay') {
